"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyPatch = exports.compare = void 0;
var index_js_1 = require("./fast-json-patch/index.cjs");
Object.defineProperty(exports, "compare", { enumerable: true, get: function () { return index_js_1.compare; } });
Object.defineProperty(exports, "applyPatch", { enumerable: true, get: function () { return index_js_1.applyPatch; } });
