"use strict";
// Auto-generated by `scripts/create-entrypoints.js`. Do not edit manually.
Object.defineProperty(exports, "__esModule", { value: true });
exports.optionalImportEntrypoints = void 0;
exports.optionalImportEntrypoints = [];
