"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.insecureHash = void 0;
var hash_js_1 = require("./js-sha1/hash.cjs");
Object.defineProperty(exports, "insecureHash", { enumerable: true, get: function () { return hash_js_1.insecureHash; } });
